$:.unshift File.expand_path('../../lib', __FILE__)

require 'pp'
require "active_support/all"

require "sms/version"
require "sms/zucp"
require "sms/taobao"

require 'redis' unless defined? Redis
require 'yaml' unless defined? YMAL

def redis
  if $redis && $redis.class.to_s =~ /Redis/
    $redis
  else
    Redis.new(host: 'localhost', port: 6379)
  end
end

def load_config
  if defined? ::Rails
    @sms_config ||= HashWithIndifferentAccess.new(YAML.load_file("#{::Rails.root}/config/sms.yml")[::Rails.env] || {})
  else
    {}
  end
end

class SmsSender

  SOURCES = %w(taobao zucp)

  attr_accessor :source, :source_config, :template_code, :attrs, :result

  def initialize(opts = {})
    self.attrs = HashWithIndifferentAccess.new(opts)

    assign_source
    extend_attrs
    return unless attrs[:source]

    send("#{attrs[:source].to_s}_init")
    attrs[:sender] = self
    @source = "Sms/#{attrs[:source]}".camelize.constantize.new(attrs)
  end

  def assign_source
    return if attrs[:source]

    attrs[:source] = redis.get('sms:source') || SOURCES.first
    return unless attrs[:source].in?(SOURCES)
  end

  def extend_attrs
    return unless load_config.present?

    self.source_config = load_config[attrs[:source]]
    return unless source_config.present?

    self.attrs = source_config.merge(attrs)
  end

  def zucp_init
    return unless attrs[:template_code].present?
    return unless source_config.present?

    templates = backend(I18n.locale => {
      'sms_zucp_templates' => source_config[:templates]
    })
    I18n.backend = I18n::Backend::Chain.new(templates)

    zucp_template_code = "sms_zucp_templates.#{attrs[:template_code].to_s.gsub('.', '_')}"
    attrs.merge!(content: I18n.t(zucp_template_code, attrs[:content_params]))
  end

  def taobao_init
    return unless attrs[:template_code].present?
    return unless source_config.present?

    template_code = attrs[:template_code].to_s.gsub('.', '_')
    taobao_template_code = source_config[:templates][template_code] rescue nil

    attrs.merge!({
      template_code: taobao_template_code,
      param: attrs[:content_params]
    })
  end

  def singleSend(number)
    return unless @source

    numbers = number_regroup(number)
    return unless numbers.present?

    @source.single_send(numbers)
  end

  def batchSend(numbers)
    return unless @source

    numbers = number_regroup(numbers)
    return unless numbers.present?

    @source.batch_send(numbers)
  end

  # 语音验证码
  def text_to_voice_verify(number)
    return unless @source

    numbers = number_regroup(number)
    return unless numbers.present?

    @source.tts_num_singlecall(numbers, 'verify')
  end

  # 语音通知
  def text_to_voice_notify(number)
    return unless @source

    numbers = number_regroup(number)
    return unless numbers.present?

    @source.tts_num_singlecall(numbers, 'notify')
  end

  def number_regroup(number)
    return number.split(',') if number.is_a?(String)
    return number if number.is_a?(Array)
    []
  end

  # Example:
  #   sms_sender = SmsSender.new
  #   sms_sender.dingtalk_call_user({
  #     staff_id: "xxxx",
  #     session: "xxxx",
  #     authed_corp_id: "xxxx",
  #     authed_staff_id: "xxxx"
  #   })
  def dingtalk_call_user(opts = {})
    @source.dingtalk_call_user(opts)
  end

  # Example:
  #   sms_sender = SmsSender.new
  #   sms_sender.dingtalk_call_set_user_list({
  #     session: "xxxx",
  #     staff_id_list: ["xxxx"]
  #   })
  def dingtalk_call_set_user_list(opts)
    @source.dingtalk_call_set_user_list(opts)
  end


  # Example:
  #   sms_sender = SmsSender.new
  #   sms_sender.info('xxxx')
  def info(msg)
    self.class.info msg
  end

  # Example:
  #   SmsSender.info('xxxx')
  def self.info(msg)
    if defined? ::Rails
      @logger ||= ActiveSupport::Logger.new("#{Rails.root}/log/sms_sender.log", "weekly")
      @logger.info "#{msg}"
    else
      puts msg
    end
  end

  protected

  def backend(translations)
    backend = I18n::Backend::Simple.new
    translations.each { |locale, data| backend.store_translations(locale, data) }
    backend
  end

end

if defined? ::Rails
  module Sms
    module Rails
      class Railtie < ::Rails::Railtie
        rake_tasks do
          # import "/Users/lihui/workplace/demo/rails/gems/sms/lib/tasks/sms.rake"
          load 'tasks/sms.rake'
        end
      end
    end
  end
end